/*
 * JFlubberMainFrm.groovy
 *
 *
 * Released under the GPL
 */

package org.bldc.jflubber

import groovy.swing.SwingXBuilder
import java.awt.Font
import javax.swing.JFileChooser
import javax.swing.ScrollPaneConstants
import javax.swing.WindowConstants
import net.miginfocom.swing.MigLayout
import org.jdesktop.swingx.JXFrame

/**
*
* @author  jwill
*/
public class JFlubberMainFrm {
    private JXFrame frame
    private clearButton, flubButton
    private javax.swing.JTextArea flubPointsTextArea
    private javax.swing.JScrollPane jScrollPane1
    private saveButton, startButton, stopButton
    private timeLabel, timer, swing, startTime = 0L, labelNo = 0
    def baseIconString = '/org/bldc/jflubber/images/'
    public JFlubberMainFrm() {
        swing = new SwingXBuilder()
        swing.lookAndFeel(new org.jdesktop.swingx.plaf.nimbus.NimbusLookAndFeel())
        swing.registerBeanFactory("woodenPane", WoodenContentPane)
        swing.registerBeanFactory("timeLabel", TimeLabel)
        frame = swing.frame(title: 'jFlubber', defaultCloseOperation: WindowConstants.EXIT_ON_CLOSE, size: [275, 460],
                resizable: false)
        initComponents()
        frame.visible = true
        //setLocationRelativeTo(null)
    }

    def clearButtonMouseReleased = {
        this.flubPointsTextArea.text = ''
        this.labelNo = 0
        this.timeLabel.text = '0:00'
    }

    def stopButtonMouseReleased = {
        if (timer != null) {
            timer.cancel();
            startTime = 0L;
        }
    }

    def flubButtonMouseReleased = {
        labelNo++;
        this.addLabel(Integer.toString(labelNo));
    }

    def saveButtonMouseReleased = {
        def chooser = swing.fileChooser();
        int rVal = chooser.showSaveDialog(frame)
        if (rVal == JFileChooser.APPROVE_OPTION) {
            def outputFile = chooser.getSelectedFile()
            try {
                outputFile.write(this.flubPointsTextArea.getText())
            } catch (IOException ex) {
                ex.printStackTrace()
            }
        }
    }

    def startButtonMouseReleased = {evt ->
        if (startTime == 0L) {
            startTime = evt.getWhen();
            timer = new Timer();
            timer.schedule(new UpdateTimeTask(timeLabel, startTime), 200, 200);
        }
    }

    def createButton(buttonString, action) {
        return swing.button(borderPainted: false, contentAreaFilled: false, focusPainted: false,
                icon: swing.imageIcon(getClass().getResource(baseIconString + buttonString + '-button.png')),
                pressedIcon: swing.imageIcon(getClass().getResource(baseIconString + buttonString + '-button-pressed.png')),
                actionPerformed: action)
    }

    String centerObject(w) {
        def width = (frame.getWidth() - w) / 2
        return width.toString()
    }

    private void initComponents() {
        def baseIconString = '/org/bldc/jflubber/images/'
        // setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        // setTitle("jFlubber");
        // setSize(300, 460)
         frame.setResizable(true);

        startButton = createButton('start', startButtonMouseReleased)

        stopButton = createButton('stop', stopButtonMouseReleased)

        flubButton = createButton('flub', flubButtonMouseReleased)
        flubButton.setFont(new Font('Dialog', 1, 18))
        flubButton.setPreferredSize(new java.awt.Dimension(90, 91))

        saveButton = createButton('save', saveButtonMouseReleased)

        clearButton = createButton('clear', clearButtonMouseReleased)

        flubPointsTextArea = swing.textArea(columns: 15, rows: 8)
        timeLabel = swing.timeLabel(text: '0:00')


        def pane = swing.woodenPane(layout: new MigLayout()) {
            def x = centerObject(261)
            widget(timeLabel, constraints: "id timeLabel, w 261!, h 81!, x " + x)
            widget(startButton, constraints: 'id startButton,x timeLabel.x-5, y timeLabel.y2+10')

            widget(stopButton, constraints: 'id stopButton, x startButton.x2+45, y startButton.y, x2 timeLabel.x2')
            flubX = centerObject(flubButton.getSize().getWidth())
            
            widget(flubButton, constraints: 'id flubButton, x 75, y 150,wrap')
            scrollPane(constraints: 'id scrollPane,x timeLabel.x+10, x2 timeLabel.x2, y flubButton.y2+10',
                    horizontalScrollBarPolicy: ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER) {
                widget(flubPointsTextArea)
            }
            widget(saveButton, constraints: 'x startButton.x, y scrollPane.y2+20')
            widget(clearButton, constraints: 'x stopButton.x, y scrollPane.y2+20, x2 timeLabel.x2')

        }
        frame.add(pane)
    }

    public void addLabel(String label) {
        def seconds = 0.0d
        if (startTime > 0L) {
            def millis = System.currentTimeMillis() - startTime
            seconds = (double) millis / 1000
        }

        flubPointsTextArea.text += "${seconds}\t${seconds}\t${label}\n"
    }
}